package main

import (
	"fmt"
	"io"
	"locking-kit-server/consts"
	"locking-kit-server/env"
	"locking-kit-server/handler"
	"locking-kit-server/utils"
	"locking-kit-server/websocket"
	"log"
	"net/http"
	"os"
	"path/filepath"
	"strconv"
	"strings"
	"time"
)

/**
 * @author yuanrh
 * @description 主函数
 * @date 2021/6/28 11:11
 **/

func main() {

	str :="2021-07-28T18:47:33+08:00"
	fmt.Println("-------------")
	fmt.Println(strings.ReplaceAll(strings.ReplaceAll(strings.ReplaceAll(strings.ReplaceAll(str,"-",""),":",""),"+0800",""),"T",""))
	fmt.Println("-------------")

	var LogFilePath = os.Getenv("APPDATA")+"\\LOCKING-KIT\\logs"
	//日志设置
	_,err := os.Stat(LogFilePath)
	if err != nil {
		os.MkdirAll(LogFilePath, os.ModePerm)
	}
	logpath :=filepath.Join(LogFilePath,fmt.Sprintf("%v-%d.log",strconv.Itoa((time.Now().Year())),time.Now().Month()))
	logFile, err := os.OpenFile(logpath, os.O_CREATE|os.O_WRONLY|os.O_APPEND, 0644)
	if err != nil {
		log.Printf("open log file failed, err:", err)
		return
	}
	multiWriter := io.MultiWriter(os.Stdout,logFile)
	log.SetOutput(multiWriter)
	log.SetPrefix(consts.LOG_PREFIX)
	log.SetFlags(log.Ldate | log.Ltime | log.Lshortfile)

	utils.InitWorker(101)
	err = env.InitWorkSpaceFromRegistry()
	if err != nil {
		log.Printf("InitWorkSpaceFromRegistry failed, err:", err)
		return
	}

	defer env.GobalFileWatch.Close()

	//http://localhost:7777/ws
	http.HandleFunc("/api/syncFolderToWorkSpace", handler.SyncFolderToWorkSpace)
	http.HandleFunc("/api/downloadFile", handler.DownloadFile)
	http.HandleFunc("/api/uploadFile", handler.UploadFile)
	http.HandleFunc("/api/getWorkSpace", handler.GetWorkSpace)
	http.HandleFunc("/api/setWorkSpace", handler.SetWorkSpace)
	http.HandleFunc("/api/restartTask", handler.RestartTask)
	http.HandleFunc("/api/login", handler.Login)
	http.HandleFunc("/api/logout", handler.Logout)
	http.HandleFunc("/api/downloadFileFromMsg", handler.DownloadFileFromMsg)
	http.HandleFunc("/api/canShowBlueOverlay", handler.CanShowBlueOverlay)
	http.HandleFunc("/api/canShowRedOverlay", handler.CanShowRedOverlay)
	
	http.HandleFunc("/websocket/subscriptionTaskSync", websocket.SubscriptionTaskSyncHandler)
	http.HandleFunc("/websocket/subscriptionMessageNotify", websocket.SubscriptionMessageNotifyHandler)

	//服务端启动
	log.Println("服务启动成功，监听端口7888，等待连接...")
	http.ListenAndServe("127.0.0.1:7888", nil)

}